<?php

namespace backend\controllers;

use Yii;
use backend\models\Users;
use backend\models\UsersSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use yii\web\Session;
/**
 * UsersController implements the CRUD actions for Users model.
 */
class UsersController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'actions' => ['test'],
                        'allow' => true,
                    ],
                    [
                        'actions' => ['update', 'index','delete','view','create'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['post'],
                ],
            ],
        ];
    }
     public function beforeAction($action)
    {
        if (!parent::beforeAction($action)) {
            return false;
        }
        // Check only when the user is logged in
        if ( !Yii::$app->user->isGuest)  { 
            
            $time=\Yii::$app->session->get('userSessionTimeout');
            
            if ($time < time()) {
                Yii::$app->user->logout();
            } else {
                \Yii::$app->session->set('userSessionTimeout',time() + Yii::$app->params['sessionTimeoutSeconds']);      
                return true; 
            }
        } else {
            return true;
        }
    }
    /**
     * Lists all Users models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new UsersSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Users model.
     * @param integer $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Users model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Users();

        if ($model->load(Yii::$app->request->post())) {
            $model->Updated_at=date('Y-m-d H:i:s');
            $model->Created_at=date('Y-m-d H:i:s');   
            $model->Password=md5($model->Password);
            //echo '<pre>';print_r($model); echo '</pre>'; die;
            if($model->save()) {     
                 Yii::$app->session->setFlash('success','User Successfully Added');
                return $this->redirect(['index']);       
               // return $this->redirect(['view', 'id' => $model->UserId]);
            }else{
                return $this->render('create', [
                'model' => $model,
                ]); 
            }
        } else {
            return $this->render('create', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Updates an existing Users model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $password=$model->Password;        
        if ($model->load(Yii::$app->request->post())) {
             $model->Updated_at=date('Y-m-d H:i:s');
             if(!empty($_POST['Users']['newPassword'])) {
                 $model->Password=md5(trim($_POST['Users']['newPassword']));
             }
             //echo '<pre>'; print_r($_POST);print_r($model); die;
            if($model->save()) {     
                 Yii::$app->session->setFlash('success','User Successfully Updated');
                return $this->redirect(['index']);       
                //return $this->redirect(['view', 'id' => $model->UserId]);
            }else{
                 return $this->render('update', [
                'model' => $model,
             ]);
            }
        } else {
            return $this->render('update', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Deletes an existing Users model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Users model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Users the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Users::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }
}
